import os
from unittest import TestCase


_ROOT = os.path.abspath(os.path.dirname(__file__))


def get_data(path):
    return os.path.join(_ROOT, 'data', path)


class SipsTestCaseBase(TestCase):
    """Base class to inherit test cases for SIPS.

    - Loads samples data from SIPS.
    """

    @staticmethod
    def load_sips(sips_file):
        return {'file': get_data(sips_file)}

    @classmethod
    def setUpClass(cls):
        cls.SIPS_DATA = {
            'IBERDROLA': cls.load_sips('HGSBKA_TXT2.TXT'),
            'HIDROCANTABRICO_PS': cls.load_sips('HIDROCANTABRICO_PS_50290_3-1A.TXT'),
            'HIDROCANTABRICO_CO': cls.load_sips('HIDROCANTABRICO_CO_50290_3-1A.TXT')
        }
        cls.SIPS_PACKED_DATA = {
            'ENDESA_PS': {'file': get_data('ERZ.INF.SEG01.ZIP')},
            'ENDESA_MEASURES': {'file': get_data('ERZ.INF2.SEG01.ZIP')},
            'HIDROCANTABRICO_PACKED_PS': {'file': get_data('HIDROCANTABRICO_PS.zip')}
        }



import codecs
import unittest

from sippers.file import SipsFile, PackedSipsFile
from sippers.utils import naturalsize
from . import SipsTestCaseBase, get_data


class PackedSipsFileTest(SipsTestCaseBase):

    def test_iterator_file(self):
        sips_file = self.SIPS_PACKED_DATA['ENDESA_PS']['file']
        psf = PackedSipsFile(sips_file)
        for sf in psf:
            for _ in sf:
                pass
            self.assertEqual(sf.stats.progress, '100%')
        self.assertEqual(psf.stats.progress, '100% (1/1)')
        sips_file = self.SIPS_PACKED_DATA['HIDROCANTABRICO_PACKED_PS']['file']
        psf = PackedSipsFile(sips_file)
        for sf in psf:
            for _ in sf:
                pass
            self.assertEqual(sf.stats.progress, '100%')
        self.assertEqual(psf.stats.progress, '100% (2/2)')


class SipsFileTest(SipsTestCaseBase):

    def test_iterator_file(self):
        for dso in self.SIPS_DATA:
            sips_file = self.SIPS_DATA[dso]['file']
            sf = SipsFile(sips_file)
            with codecs.open(self.SIPS_DATA[dso]['file'],
                             encoding=sf.parser.encoding) as orig:
                orig_content = orig.read()
            content = ''
            for line in sf:
                content += line['orig']
            self.assertEqual(orig_content, content)
            sf.close()

    def test_stats(self):
        for dso in self.SIPS_DATA:
            sips_file = self.SIPS_DATA[dso]['file']
            line_numbers = len(open(sips_file, 'r').readlines())
            path = get_data(sips_file)
            sf = SipsFile(path)
            for _ in sf:
                read = sf.stats.read
                progress = sf.stats.progress
            self.assertEqual(progress, '100%')
            self.assertEqual(sf.stats.line_number, line_numbers)
            self.assertEqual(naturalsize(read), sf.stats.size)
            sf.close()

    def test_with_statement(self):
        sips_file = self.SIPS_DATA.values()[0]['file']
        path = get_data(sips_file)
        with SipsFile(path) as sf:
            for _ in sf:
                pass
        self.assertIs(sf.fd.closed, True)

    def test_resume(self):
        sips_file = self.SIPS_DATA['IBERDROLA']['file']
        path = get_data(sips_file)
        sf = SipsFile(path)
        with codecs.open(sips_file, encoding=sf.parser.encoding) as orig:
            orig_content = orig.read()
        content = ''
        for idx, line in enumerate(sf):
            content += line['orig']
            if idx == 5:
                break
        sf.close()
        stats = sf.stats
        sf = SipsFile(path, resume=stats)
        self.assertEqual(sf.resume_line_number, 6)
        for line in sf:
            content += line['orig']
        sf.close()
        self.assertEqual(orig_content, content)



from __future__ import absolute_import

import csv
import StringIO

from sippers import logger
from sippers.utils import build_dict
from sippers.parsers.parser import Parser, register
from sippers.adapters.cnmc import CnmcSipsAdapter, CnmcMeasuresAdapter
from sippers.models.cnmc import CnmcMeasuresSchema

class Cnmc(Parser):

    # En aquest cas els llegim amb el csv.DictReader en comptes de fer-ho amb
    # un Schema de marshmallow, ja que el csv pot contenir comes dins d'un
    # camp si van wrapped entre cometes i el marshmallow aixo no ho contempla.

    # amb csv.DictReader obtindrem un diccionari amb els headers_ps
    # que li indiquem aqui, en comptes d'anar-los a buscar a l'Schema.
    # El diccionari el podem utilitzar per passar-li al
    # self.adapter.load igual que el resultat de l'esquema

    pattern = '[0-9]{4}-[0-9]{2}-[0-9]{2}_electricidad_sips.csv'
    encoding = "UTF-8"

    def __init__(self, strict=False):

        # l'ordre dels camps segons format cnmc
        self.headers_ps = [
                'cod_distri',
                'name',
                'codi_postal',
                'direccio',
                'poblacio',
                'data_alta',
                'tarifa',
                'tensio',
                'pot_max_bie',
                'pot_max_puesta',
                'tipo_pm',
                'indicatiu_icp',
                'perfil_consum',
                'der_extensio',
                'der_acces_llano', # la dada no sabem si es llano, valle o punta
                'propietat_equip_mesura',
                'propietat_icp',
                'potencies_contractades', # n periodes seguits separats per ';'
                'data_ulti_mov',
                'data_ult_canv',
                'data_lim_exten',
                'data_ult_lect',
                'impago',
                'fianza',
                'tipo_id_titular',
                'id_titular',
                'nom_complet',
                'direccio_titular',
                'primera_vivenda'
            ]
        self.adapter = CnmcSipsAdapter(strict=strict)

    def parse_line(self, line):

        # passar previament la linia pel csv reader
        # per que agafi be els camps tot i les comes dins del camp direccio
        # per fer-ho cal passar-la a StringIO
        l = StringIO.StringIO(line)
        reader = csv.DictReader(l, fieldnames=self.headers_ps, delimiter=',')
        linia = reader.next() # nomes n'hi ha una

        parsed = {'ps': {}, 'orig': line}
        result, errors = self.adapter.load(linia)

        if errors:
            logger.error(errors)
        parsed['ps'] = result
        return parsed, errors

register(Cnmc)

class CnmcCons(Parser):

    # En el cas de les mesures, usem Schema per mantenir el format i
    # perque no hi trobarem mes comes que les delimiters
    pattern = '[0-9]{4}-[0-9]{2}-[0-9]{2}_electricidad_consumos.csv'
    encoding = "UTF-8"
    delimiter = ','

    def __init__(self, strict=False):
        self.schema = CnmcMeasuresSchema(strict=strict)
        self.adapter = CnmcMeasuresAdapter(strict=strict)
        self.measures_adapter = self.adapter
        self.fields = []
        self.headers = []
        for f in sorted(self.schema.fields,
                key=lambda f: self.schema.fields[f].metadata['position']):
            field = self.schema.fields[f]
            self.fields.append((f, field.metadata))
            self.headers.append(f)

    def parse_line(self, line):
        slinia = tuple(line.split(self.delimiter))
        slinia = map(lambda s: s.strip(), slinia)
        parsed = {'ps': {}, 'measure_cnmc': [], 'orig': line}
        all_errors = {}
        consums = build_dict(self.headers, slinia)
        result, errors = self.adapter.load(consums)
        if errors:
            logger.error(errors)
            all_errors.update(errors)
        parsed['measure_cnmc'] = result

        return parsed, errors

register(CnmcCons)



from marshmallow import Schema, fields
from marshmallow.validate import OneOf

TARIFFS = [
    '2.0A',
    '2.0DHA',
    '2.0DHS',
    '2.1A',
    '2.1DHA',
    '2.1DHS',
    '3.0A',
    '3.1A',
    '6.1',
    '6.1A',
    '6.1B',
    '6.2',
    '6.3',
    '6.4',
    '6.5'
]

TYPE_PS = ['01', '02', '03', '04', '05']

CONSUMPTION_PROFILE = ['Pa', 'Pb', 'Pc', 'Pd']


class Document(object):
    """Document object

    :param data: Data parsed.
    :param adapter: Adapter used to parse this data.

    This document is used to encapsulated a object in
    :func:`sippers.parsers.parser.Parser.parse_line`
    """
    def __init__(self, data, adapter):
        self.data = data
        self.adapter = adapter
        self.backend = None

    @property
    def backend_data(self):
        """Get data after using the filter :func:`sippers.adapters.pre_insert`
        """
        if not self.backend:
            raise Exception("No backend defined")
        self.adapter.backend = self.backend
        return self.adapter._invoke_processors(
            'pre_insert', False, self.data, False
        )


class SipsSchema(Schema):
    """Base model for SIPS
    """
    name = fields.String(required=True)
    ref = fields.String()
    distri = fields.String()
    cod_distri = fields.String()
    poblacio = fields.String()
    direccio = fields.String()
    codi_postal = fields.String()
    provincia = fields.String()
    data_alta = fields.DateTime(allow_none=True)
    nom = fields.String()
    cognom = fields.String()
    direccio_titular = fields.String()
    pot_max_puesta = fields.Float()
    pot_max_bie = fields.Float()
    tarifa = fields.String(validate=OneOf(TARIFFS))
    des_tarifa = fields.String()
    tensio = fields.String()
    tipo_pm = fields.String(validate=OneOf(TYPE_PS))
    indicatiu_icp = fields.String(validate=OneOf(['0', '1']), allow_none=True)
    perfil_consum = fields.String(
        validate=OneOf(CONSUMPTION_PROFILE), allow_none=True
    )
    der_extensio = fields.Float()
    der_acces_llano = fields.Float()
    der_acces_valle = fields.Float()
    propietat_equip_mesura = fields.String(validate=OneOf(['0', '1']))
    propietat_icp = fields.String(validate=OneOf(['0', '1'])),
    pot_cont_p1 = fields.Float()
    pot_cont_p2 = fields.Float()
    pot_cont_p3 = fields.Float()
    pot_cont_p4 = fields.Float()
    pot_cont_p5 = fields.Float()
    pot_cont_p6 = fields.Float()
    data_ulti_mov = fields.DateTime(allow_none=True)
    data_ult_canv = fields.DateTime(allow_none=True)
    data_lim_exten = fields.DateTime(allow_none=True)
    persona_fj = fields.String(validate=OneOf(['0', '1']), allow_none=True)
    primera_vivenda = fields.String(validate=OneOf(['0', '1']), allow_none=True)
    fianza = fields.Float()


class MeasuresSchema(Schema):
    """Base model for measures.
    """
    name = fields.String(required=True)
    data_final = fields.DateTime(allow_none=True)
    real_estimada = fields.String()
    activa_1 = fields.Integer()
    tipo_a1 = fields.String()
    activa_2 = fields.Integer()
    tipo_a2 = fields.String()
    activa_3 = fields.Integer()
    tipo_a3 = fields.String()
    activa_4 = fields.Integer()
    tipo_a4 = fields.String()
    activa_5 = fields.Integer()
    tipo_a5 = fields.String()
    activa_6 = fields.Integer()
    tipo_a6 = fields.String()
    reactiva_1 = fields.Integer()
    tipo_r1 = fields.String()
    reactiva_2 = fields.Integer()
    tipo_r2 = fields.String()
    reactiva_3 = fields.Integer()
    tipo_r3 = fields.String()
    reactiva_4 = fields.Integer()
    tipo_r4 = fields.String()
    reactiva_5 = fields.Integer()
    tipo_r5 = fields.String()
    reactiva_6 = fields.Integer()
    tipo_r6 = fields.String()
    potencia_1 = fields.Float()
    tipo_p1 = fields.String()
    potencia_2 = fields.Float()
    tipo_p2 = fields.String()
    potencia_3 = fields.Float()
    tipo_p3 = fields.String()
    potencia_4 = fields.Float()
    tipo_p4 = fields.String()
    potencia_5 = fields.Float()
    tipo_p5 = fields.String()
    potencia_6 = fields.Float()
    tipo_p6 = fields.String()



from sippers.adapters import SipsAdapter, MeasuresAdapter
from sippers.models.cnmc_v2 import CnmcV2SipsSchema, CnmcV2MeasuresSchema
from marshmallow import pre_load, fields


class CnmcV2SipsAdapter(SipsAdapter, CnmcV2SipsSchema):
    '''A self.fields tenim els camps per defecte, els de SipsSchema base'''

    @pre_load
    def add_distri_description(self, data):
        cod_distri = data.get('codigoEmpresaDistribuidora')
        if cod_distri == '059':
            data['nombreEmpresaDistribuidora'] = 'GRUPO ELECTRIFICACION RURAL BINEFAR'
        return data

    @pre_load
    def parse_selections(self, data):
        selection_fields = [
            'codigoTarifaATREnVigor',
            'codigoTipoSuministro',
            'codigoPropiedadICP',
            'codigoDHEquipoDeMedida',
            'codigoTensionMedida',
            'tipoIdTitular',
            'codigoClasificacionPS',
            'codigoTelegestion',
            'codigoTensionV',
            'codigoTipoContrato',
            'codigoPeriodicidadFacturacion',
            'codigoPropiedadEquipoMedida'
         ]
        for select_field in selection_fields:
            if data.get(select_field) == '':
                data[select_field] = None
        return data

    @pre_load
    def fix_dates(self, data):
        for attr, field in self.fields.iteritems():
            if isinstance(field, fields.DateTime):
                # si ve alguna data, assumim que ve correcta
                if data[attr] == u'':
                    data[attr] = None
                else:
                    data[attr] += 'T00:00:00'
        return data

class CnmcV2MeasuresAdapter(MeasuresAdapter, CnmcV2MeasuresSchema):

    @pre_load
    def parse_selections(self, data):
        selection_fields = [
            'codigoTarifaATR',
            'codigoDHEquipoDeMedida',
            'codigoTipoLectura'
        ]
        for select_field in selection_fields:
            if data.get(select_field) == '':
                data[select_field] = None
        return data

    @pre_load
    def fix_dates(self, data):
        for attr, field in self.fields.iteritems():
            if isinstance(field, fields.DateTime):
                # si ve alguna data, assumim que ve correcta
                if data[attr] == u'':
                    data[attr] = None
                else:
                    data[attr] += 'T00:00:00'
        return data


from marshmallow import Schema, fields
from marshmallow.validate import OneOf

from sippers.models import TYPE_PS, CONSUMPTION_PROFILE


TARIFFS_OCSUM = {
    '001': "2.0A",
    '003': "3.0A",
    '004': "2.0DHA",
    '005': "2.1A",
    '006': "2.1DHA",
    '007': "2.0DHS",
    '008': "2.1DHS",
    '011': "3.1A",
    '012': "6.1",
    '013': "6.2",
    '014': "6.3",
    '015': "6.4",
    '016': "6.5"
}

class IberdrolaSipsSchema(Schema):

    name = fields.String(position=0, length=22)
    direccio = fields.String(position=1, length=150)
    poblacio = fields.String(position=2, length=45)
    codi_postal = fields.String(position=3, length=10)
    provincia = fields.String(position=4, length=45)
    persona_fj = fields.String(
        validate=OneOf(['PF', 'PJ']), position=5, length=2
    )
    cognom = fields.String(position=6, length=50)
    direccio_titular = fields.String(position=7, length=80)
    municipi_titular = fields.String(position=8, length=45)
    codi_postal_titular = fields.String(position=9, length=10)
    provincia_titular = fields.String(position=10, length=45)
    data_alta = fields.Date(position=11, length=10)
    tarifa = fields.String(
        validate=OneOf(TARIFFS_OCSUM.keys()), position=12, length=3
    )
    tensio = fields.Integer(position=13, length=9)
    pot_max_bie = fields.Float(position=14, length=12)
    pot_max_puesta = fields.Float(position=15, length=12)
    tipo_pm = fields.String(validate=OneOf(TYPE_PS), position=16, length=2)
    indicatiu_icp = fields.String(position=17, length=1)
    perfil_consum = fields.String(
        validate=OneOf(CONSUMPTION_PROFILE), position=18, length=2
    )
    der_acces_reconocido = fields.Float(position=19, length=12)
    der_extensio = fields.Float(position=20, length=12)
    propietat_equip_mesura = fields.String(position=21, length=1)
    propietat_icp = fields.String(position=22, length=1)
    pot_cont_p1 = fields.Float(position=23, length=12)
    pot_cont_p2 = fields.Float(position=24, length=12)
    pot_cont_p3 = fields.Float(position=25, length=12)
    pot_cont_p4 = fields.Float(position=26, length=12)
    pot_cont_p5 = fields.Float(position=27, length=12)
    pot_cont_p6 = fields.Float(position=28, length=12)
    pot_cont_p7 = fields.Float(position=29, length=12)
    pot_cont_p8 = fields.Float(position=30, length=12)
    pot_cont_p9 = fields.Float(position=31, length=12)
    pot_cont_p10 = fields.Float(position=32, length=12)
    data_ult_canv = fields.Date(position=33, length=10)
    data_ulti_mov = fields.Date(position=34, length=10)
    data_lim_exten = fields.Date(position=35, length=10)
    data_ult_lect = fields.Date(position=36, length=10)
    pot_disp_caixa = fields.Float(position=37, length=12)
    impago = fields.String(position=38, length=11)
    diposit_garantia = fields.String(
        validate=OneOf(['1', '2']), position=39, length=1
    )
    import_diposit = fields.Float(position=40, length=11)
    primera_vivenda = fields.String(
        validate=OneOf(['S', 'N']), position=41, length=1
    )
    telegest_actiu = fields.String(position=42, length=2)
    any_sub = fields.Integer(position=43, length=4)
    trimestre_sub = fields.Integer(position=44, length=1)


class IberdrolaMeasuresSchema(Schema):
    any_consum = fields.Integer(position=0, length=4)
    facturacio_consum = fields.String(position=1, length=4)
    data_anterior = fields.Date(position=2, length=10)
    data_final = fields.Date(position=3, length=10)
    tarifa_consums = fields.String(position=4, length=4)
    DH = fields.String(position=5, length=3)
    activa_1 = fields.Integer(position=6, length=14)
    activa_2 = fields.Integer(position=7, length=14)
    activa_3 = fields.Integer(position=8, length=14)
    activa_4 = fields.Integer(position=9, length=14)
    activa_5 = fields.Integer(position=10, length=14)
    activa_6 = fields.Integer(position=11, length=14)
    activa_7 = fields.Integer(position=12, length=14)
    reactiva_1 = fields.Integer(position=13, length=14)
    reactiva_2 = fields.Integer(position=14, length=14)
    reactiva_3 = fields.Integer(position=15, length=14)
    reactiva_4 = fields.Integer(position=16, length=14)
    reactiva_5 = fields.Integer(position=17, length=14)
    reactiva_6 = fields.Integer(position=18, length=14)
    reactiva_7 = fields.Integer(position=19, length=14)
    potencia_1 = fields.Integer(position=20, length=11)
    potencia_2 = fields.Integer(position=21, length=11)
    potencia_3 = fields.Integer(position=22, length=11)
    potencia_4 = fields.Integer(position=23, length=11)
    potencia_5 = fields.Integer(position=24, length=11)
    potencia_6 = fields.Integer(position=25, length=11)
    potencia_7 = fields.Integer(position=26, length=11)



from __future__ import absolute_import

from sippers import logger
from sippers.utils import build_dict
from sippers.adapters.endesa import EndesaSipsAdapter, EndesaMeasuresAdapter
from sippers.models.endesa import EndesaSipsSchema, EndesaMeasuresSchema
from sippers.parsers.parser import Parser, register


class Endesa(Parser):

    pattern = '(SEVILLANA|FECSA|ERZ|UNELCO|GESA).INF.SEG0[1-5].(zip|ZIP)$'
    encoding = "iso-8859-15"
    delimiter = ';'

    def __init__(self, strict=False):
        self.adapter = EndesaSipsAdapter(strict=strict)
        self.schema = EndesaSipsSchema(strict=strict)
        self.fields_ps = []
        self.headers_ps = []
        for f in sorted(self.schema.fields,
                key=lambda f: self.schema.fields[f].metadata['position']):
            field = self.schema.fields[f]
            self.fields_ps.append((f, field.metadata))
            self.headers_ps.append(f)

        self.fields = self.fields_ps

    def parse_line(self, line):
        slinia = tuple(unicode(line.decode(self.encoding)).split(self.delimiter))
        slinia = map(lambda s: s.strip(), slinia)
        parsed = {'ps': {}, 'measures': {}, 'orig': line}
        data = build_dict(self.headers_ps, slinia)
        result, errors = self.adapter.load(data)
        if errors:
            logger.error(errors)
        parsed['ps'] = result
        return parsed, errors


register(Endesa)


class EndesaCons(Parser):

    delimiter = ';'
    pattern = '(SEVILLANA|FECSA|ERZ|UNELCO|GESA).INF2.SEG0[1-5].(zip|ZIP)$'
    encoding = "iso-8859-15"

    def __init__(self, strict=False):
        self.schema = EndesaMeasuresSchema(strict=strict)
        self.adapter = EndesaMeasuresAdapter(strict=strict)
        self.measures_adapter = self.adapter
        self.fields = []
        self.headers = []
        for f in sorted(self.schema.fields,
                key=lambda f: self.schema.fields[f].metadata['position']):
            field = self.schema.fields[f]
            self.fields.append((f, field.metadata))
            self.headers.append(f)
        self.measures_start = 1
        self.measures_step = len(self.headers) - self.measures_start

    def parse_line(self, line):
        slinia = tuple(line.split(self.delimiter))
        slinia = map(lambda s: s.strip(), slinia)
        start = self.measures_start
        step = self.measures_step
        parsed = {'ps': {}, 'measures': [], 'orig': line}
        c_line = slinia[start:start+step]
        all_errors = {}
        while c_line:
            c_line.insert(0, slinia[0])
            consums = build_dict(self.headers, c_line)
            result, errors = self.adapter.load(consums)
            if errors:
                logger.error(errors)
                all_errors.update(errors)
            parsed['measures'].append(result)
            start += step
            c_line = slinia[start:start+step]
        return parsed, errors


register(EndesaCons)



from __future__ import absolute_import

from urlparse import urlparse as std_urlparse
from sippers.backends.base import BaseBackend

_AVAILABLE_BACKENDS = {}


def register(name, cls):
    """Register a backend

    Use this function to register a Backend class for and schema. If you want
    to register your own backend you can do::

        class YourAwesomeBackend(BaseBackend):
            pass

        register('awesome', YourAwsomeBackend)

    Then with the URL ``awesome://user:pass@host/db`` with the function
    :func:`get_backend`

    :param name: Backend name
    :param class cls: Backend class
    """
    _AVAILABLE_BACKENDS[name] = cls


def urlparse(url):
    url = std_urlparse(url)
    config = {
        'backend': url.scheme,
        'username': url.username,
        'password': url.password,
        'hostname': url.hostname,
        'port': url.port,
        'db': url.path.lstrip('/')
    }
    return config


def get_backend(url):
    """Get the backend class by and URL.

    :param url: URL for identify a backend.
    """
    config = urlparse(url)
    backend = config['backend']
    if backend not in _AVAILABLE_BACKENDS:
        raise Exception(
            'Backend {} is not available/registered'.format(backend)
        )
    return _AVAILABLE_BACKENDS[backend]

# Import Backends
from sippers.backends.mongodb import MongoDBBackend


from . import SipsTestCaseBase
from sippers.parsers.parser import get_parser
from sippers.file import SipsFile, PackedSipsFile


COMPANY_PARSERS = {
    "IBERDROLA": "sippers.parsers.iberdrola.Iberdrola",
    "HIDROCANTABRICO_PS": "sippers.parsers.hidrocantabrico.Hidrocantabrico",
    "HIDROCANTABRICO_CO": "sippers.parsers.hidrocantabrico.HidrocantabricoMeasures",
    "ENDESA_PS": "sippers.parsers.endesa.Endesa",
    "ENDESA_MEASURES": "sippers.parsers.endesa.EndesaCons",
    "HIDROCANTABRICO_PACKED_PS": "sippers.parsers.hidrocantabrico.Hidrocantabrico"
}


class TestSipsDetection(SipsTestCaseBase):
    def test_name_pattern(self):
        for dso in self.SIPS_DATA:
            sips_file = self.SIPS_DATA[dso]['file']
            parser = get_parser(sips_file)
            path = '.'.join([parser.__module__, parser.__name__])
            self.assertEquals(
                COMPANY_PARSERS[dso],
                path
            )

    def test_assign_parser(self):
        for dso in self.SIPS_DATA:
            sips_file = self.SIPS_DATA[dso]['file']
            sips = SipsFile(sips_file)
            parser = sips.parser.__class__
            path = '.'.join([parser.__module__, parser.__name__])
            self.assertEquals(
                COMPANY_PARSERS[dso],
                path
            )
            sips.close()
        for dso in self.SIPS_PACKED_DATA:
            sips_file = self.SIPS_PACKED_DATA[dso]['file']
            sips = PackedSipsFile(sips_file)
            parser = sips.parser.__class__
            path = '.'.join([parser.__module__, parser.__name__])
            self.assertEquals(
                COMPANY_PARSERS[dso],
                path
            )
            sips.close()


from marshmallow import Schema, fields


class EndesaSipsSchema(Schema):
    name = fields.String(position=0)
    distri = fields.String(position=1)
    cod_distri = fields.String(position=2)
    direccio = fields.String(position=3)
    poblacio = fields.String(position=4)
    codi_postal = fields.String(position=5)
    provincia = fields.String(position=6)
    data_alta = fields.String(position=7)
    tarifa = fields.String(position=8)
    des_tarifa = fields.String(position=9)
    tensio = fields.String(position=10)
    pot_max_bie = fields.String(position=11)
    pot_max_puesta = fields.String(position=12)
    tipo_pm = fields.String(position=13)
    indicatiu_icp = fields.String(position=14)
    perfil_consum = fields.String(position=15)
    der_extensio = fields.String(position=16)
    der_acces_llano = fields.String(position=17)
    der_acces_valle = fields.String(position=18)
    propietat_equip_mesura = fields.String(position=19)
    propietat_icp = fields.String(position=20)
    pot_cont_p1 = fields.String(position=21)
    pot_cont_p2 = fields.String(position=22)
    pot_cont_p3 = fields.String(position=23)
    pot_cont_p4 = fields.String(position=24)
    pot_cont_p5 = fields.String(position=25)
    pot_cont_p6 = fields.String(position=26)
    data_ulti_mov = fields.String(position=27)
    data_ult_canv = fields.String(position=28)
    data_lim_exten = fields.String(position=29)
    data_ult_lect = fields.String(position=30)
    talls = fields.String(position=31)
    fianza = fields.String(position=32)
    persona_fj = fields.String(position=33)
    nom = fields.String(position=34)
    cognom = fields.String(position=35)
    direccio_titular = fields.String(position=36)
    municipi_titular = fields.String(position=37)
    codi_postal_titular = fields.String(position=38)
    provincia_titular = fields.String(position=39)
    primera_vivenda = fields.String(position=40)
    facturacio = fields.String(position=41)
    salt = fields.String(position=42)


class EndesaMeasuresSchema(Schema):
    name = fields.String(position=0)
    data_final = fields.String(position=1)
    real_estimada = fields.String(position=2)
    tipo_a1 = fields.String(position=3)
    activa_1 = fields.Integer(position=4)
    tipo_a2 = fields.String(position=5)
    activa_2 = fields.Integer(position=6)
    tipo_a3 = fields.String(position=7)
    activa_3 = fields.Integer(position=8)
    tipo_a4 = fields.String(position=9)
    activa_4 = fields.Integer(position=10)
    tipo_a5 = fields.String(position=11)
    activa_5 = fields.Integer(position=12)
    tipo_a6 = fields.String(position=13)
    activa_6 = fields.Integer(position=14)
    tipo_r1 = fields.String(position=15)
    reactiva_1 = fields.Integer(position=16)
    tipo_r2 = fields.String(position=17)
    reactiva_2 = fields.Integer(position=18)
    tipo_r3 = fields.String(position=19)
    reactiva_3 = fields.Integer(position=20)
    tipo_r4 = fields.String(position=21)
    reactiva_4 = fields.Integer(position=22)
    tipo_r5 = fields.String(position=23)
    reactiva_5 = fields.Integer(position=24)
    tipo_r6 = fields.String(position=25)
    reactiva_6 = fields.Integer(position=26)
    tipo_p1 = fields.String(position=27)
    potencia_1 = fields.Integer(position=28)
    tipo_p2 = fields.String(position=29)
    potencia_2 = fields.Integer(position=30)
    tipo_p3 = fields.String(position=31)
    potencia_3 = fields.Integer(position=32)
    tipo_p4 = fields.String(position=33)
    potencia_4 = fields.Integer(position=34)
    tipo_p5 = fields.String(position=35)
    potencia_5 = fields.Integer(position=36)
    tipo_p6 = fields.String(position=37)
    potencia_6 = fields.Integer(position=38)


import json

from sippers import get_data
from sippers.adapters import SipsAdapter, MeasuresAdapter, pre_insert
from sippers.models import SipsSchema, MeasuresSchema, TARIFFS as BASE_TARIFFS
from marshmallow import pre_load, fields


TARIFFS = {
    '20A': '2.0A',
    '20DHA': '2.0DHA',
    '20DHS': '2.0DHS',
    '21A': '2.1A',
    '21DHA': '2.1DHA',
    '21DHS': '2.1DHS',
    '30A': '3.0A',
    '31A': '3.1A',
    '61': '6.1',
    '61A': '6.1A',
    '61B': '6.1B',
    '62': '6.2',
    '63': '6.3',
    '64': '6.4',
    '65': '6.5'
}


class HidrocantabricoSipsAdapter(SipsAdapter, SipsSchema):

    with open(get_data('hc_poblacions.json'), 'r') as f:
        poblacions = json.load(f, encoding="utf-8")

    @pre_load
    def adapt_tarifa(self, data):
        tarifa = data.get('tarifa')
        if tarifa not in BASE_TARIFFS:
            tarifa = TARIFFS.get(tarifa, tarifa)
        data['tarifa'] = tarifa
        return data

    @pre_load
    def fix_floats(self, data):
        for attr, field in self.fields.iteritems():
            if isinstance(field, fields.Float):
                if not data.get(attr):
                    data[attr] = '0'
                data[attr] = data[attr].replace(',', '.')
        return data

    @pre_load
    def fix_poblacio(self, data):
        data['poblacio'] = self.poblacions.get(data['poblacio'], None)
        return data

    @pre_load
    def fix_primera_vivenda(self, data):
        mapping = {'N': '0', 'S': '1'}
        pv = data.get('primera_vivenda')
        data['primera_vivenda'] = mapping.get(pv)
        return data

    @pre_load
    def fix_dates(self, data):
        for attr, field in self.fields.iteritems():
            if isinstance(field, fields.DateTime):
                orig = data.get(attr)
                if orig not in ('', '0', '00000000'):
                    data[attr] = '{}-{}-{}T00:00:00'.format(
                        orig[0:4], orig[4:6], orig[6:8]
                    )
                else:
                    data[attr] = None
        return data

    @pre_load
    def fix_perfil_consum(self, data):
        pc = data.get('perfil_consum')
        if pc:
            data['perfil_consum'] = 'P' + pc.lower()
        else:
            data['perfil_consum'] = None
        return data

    @pre_load
    def fix_indicatiu_icp(self, data):
        icp = data.get('indicatiu_icp')
        mapping = {'S': '1', 'N': '0'}
        data['indicatiu_icp'] = mapping.get(icp)
        return data

    @pre_load
    def fix_propietat_equip_mesura(self, data):
        mapping = {'D': '0', 'T': '1'}
        propietat = data.get('propietat_equip_mesura')
        data['propietat_equip_mesura'] = mapping.get(propietat)
        return data

    @pre_load
    def fix_persona_fj(self, data):
        mapping = {'F': '0', 'J': '1'}
        fj = data.get('persona_fj')
        data['persona_fj'] = mapping.get(fj)
        return data


class HidrocantabricoMeasuresAdapter(MeasuresAdapter, MeasuresSchema):

    @pre_load
    def fix_numbers(self, data):
        for attr, field in self.fields.iteritems():
            if isinstance(field, (fields.Integer, fields.Float)):
                if not data.get(attr):
                    data[attr] = 0
                else:
                    data[attr] = float(data.get(attr).replace(',', '.'))
        return data

    @pre_load
    def fix_dates(self, data):
        for attr, field in self.fields.iteritems():
            if isinstance(field, fields.DateTime):
                orig = data.get(attr)
                if orig not in ('', '0', '00000000'):
                    data[attr] = '{}-{}-{}T00:00:00'.format(
                        orig[0:4], orig[4:6], orig[6:8]
                    )
                else:
                    data[attr] = None
        return data

    @pre_insert
    def fix_name(self, data):
        backend = self.backend
        result = backend.get(self.backend.ps_collection, {
            'ref': data['name'], 'cod_distri': '0026'}
        )
        if result:
            data['name'] = result[0]['name']
        return data



from . import SipsTestCaseBase
from sippers.file import SipsFile, PackedSipsFile


class TestRegisteredClasses(SipsTestCaseBase):
    def test_registered_class(self):
        from sippers.parsers.parser import _PARSERS

        self.assertItemsEqual(_PARSERS.keys(), [
            'sippers.parsers.iberdrola.Iberdrola',
            'sippers.parsers.endesa.Endesa',
            'sippers.parsers.endesa.EndesaCons',
            'sippers.parsers.hidrocantabrico.Hidrocantabrico',
            'sippers.parsers.hidrocantabrico.HidrocantabricoMeasures',
            'sippers.parsers.cnmc.Cnmc',
            'sippers.parsers.cnmc.CnmcCons',
        ])


class TestParser(SipsTestCaseBase):
    def test_parse_ps(self):
        for dso in self.SIPS_DATA:
            sips_file = self.SIPS_DATA[dso]['file']
            lines = []
            line_numbers = len(open(sips_file, 'r').readlines())
            with SipsFile(sips_file, strict=True) as sf:
                for line in sf:
                    self.assertIn('ps', line)
                    self.assertIn('measures', line)
                    self.assertIn('orig', line)
                    lines.append(line['ps'])
            self.assertEqual(len(lines), line_numbers)
        for dso in self.SIPS_PACKED_DATA:
            sips_file = self.SIPS_PACKED_DATA[dso]['file']
            with PackedSipsFile(sips_file, strict=True) as psf:
                for sf in psf:
                    lines = []
                    for line in sf:
                        self.assertIn('ps', line)
                        self.assertIn('measures', line)
                        self.assertIn('orig', line)
                        lines.append(line['ps'])


# -*- coding: utf-8 -*-
from __future__ import absolute_import
from datetime import datetime
import os
import re

from sippers import logger
from sippers.exceptions import ParserNotFoundException


def parse_datetime(value, dataformat):
    # Funcio per l'add_formatter converteixi de string a datetime
    try:
        res = datetime.strptime(value, dataformat)
    except:
        res = None
    return res


def parse_float(value):
    # Funcio per l'add_formatter converteixi valors en coma a float amb punt
    try:
        punts = value.replace(',', '.')
        deci = punts.split('.')[-1]
        nume = punts.split('.')[:-1]
        if nume:
            res = float('{}.{}'.format(''.join(nume), deci))
        else:
            res = value
    except:
        res = None
    return res


def slices(s, *args):
    # Funció per tallar un string donades les mides dels camps.
    # La llista amb les mides dels camps entren per els args.
    position = 0
    for length in args:
        yield s[position:position + length]
        position += length

MAGNITUDS = {
    'Wh': 1000,
    'kWh': 1
}


_PARSERS = {

}


def register(cls):
    """Register a parser

    :type cls: Parser class
    """
    module = cls.__module__
    path = '.'.join([module, cls.__name__])
    _PARSERS[path] = cls


def get_parser(sips_file):
    for path, cls in _PARSERS.items():
        if cls.detect(sips_file):
            return cls
    logger.error("Parser not found for file %s", sips_file)
    raise ParserNotFoundException()


class Parser(object):
    """Base parser interface.
    """

    encoding = "iso-8859-15"

    @classmethod
    def detect(cls, sips_file):
        if cls.pattern:
            return re.match(cls.pattern, os.path.basename(sips_file))
        return False

    def parse_line(self, line):
        """Parse a line of a SIPS file.

        :param line: line of the file
        """
        raise NotImplementedError("Should have implemented this")






from __future__ import absolute_import
import zipfile

import click
from sippers.file import SipsFile, PackedSipsFile
from sippers.backends import get_backend

@click.group()
def sippers():
    pass



@sippers.command(name="import")
@click.option('--file', help="SIPS File to import", type=click.Path(exists=True), required=True)
@click.option('--backend', help="Backend url", required=True)
def import_file(file, backend):
    url = backend
    backend = get_backend(backend)
    with backend(url) as bnd:
        if zipfile.is_zipfile(file):
            click.echo("Using packed SIPS File for {}".format(file))
            with PackedSipsFile(file) as psf:
                pstats = psf.stats
                for sips_file in psf:
                    print sips_file.path
                    stats = sips_file.stats
                    for line in sips_file:
                        if not line:
                            continue
                        bnd.insert(line)
                        print pstats.progress, stats.progress, stats.elapsed_time, stats.speed
        else:
            with SipsFile(file) as sips_file:
                stats = sips_file.stats
                for line in sips_file:
                    if not line:
                        continue
                    bnd.insert(line)
                    print stats.progress, stats.elapsed_time, stats.speed



if __name__ == "__main__":
    sippers()




from sippers.adapters import SipsAdapter, MeasuresAdapter
from sippers.models import SipsSchema, MeasuresSchema
from sippers.models.iberdrola import TARIFFS_OCSUM
from marshmallow import pre_load, fields


class IberdrolaSipsAdapter(SipsAdapter, SipsSchema):

    @pre_load
    def fix_dates(self, data):
        for attr, field in self.fields.iteritems():
            if isinstance(field, fields.DateTime):
                data[attr] += 'T00:00:00'
        return data

    @pre_load
    def adapt_distribuidora(self, data):
        data['distri'] = 'IBERDROLA'
        data['cod_distri'] = '0021'
        return data

    @pre_load
    def fix_floats(self, data):
        for attr, field in self.fields.iteritems():
            if isinstance(field, fields.Float):
                if not data.get(attr):
                    data[attr] = 0
                if isinstance(data[attr], basestring):
                    data[attr] = data[attr].replace(',', '.')
        return data

    @pre_load
    def adapt_indicatiu_icp(self, data):
        if data.get('indicatiu_icp') == '2':
            data['indicatiu_icp'] = '1'
        return data

    @pre_load
    def adapt_persona_fisica_juridica(self, data):
        persona = data.get('persona_fj')
        persona_map = {'PJ': '1', 'PF': '0'}
        if persona and persona in persona_map:
            data['persona_fj'] = persona_map[persona]
        return data

    @pre_load
    def adapt_tarifa(self, data):
        tarifa = data.get('tarifa')
        if tarifa and tarifa in TARIFFS_OCSUM:
            data['tarifa'] = TARIFFS_OCSUM[tarifa]
        return data

    @pre_load
    def adapt_tensio(self, data):
        tensio = data.get('tensio')
        try:
            tensio = int(tensio)
        except ValueError:
            pass
        data['tensio'] = str(tensio)
        return data

    @pre_load
    def adapt_primera_vivenda(self, data):
        pv_map = {u'N': '0', u'S': '1'}
        pv = data.get('primera_vivenda')
        if pv and pv in pv_map:
            data['primera_vivenda'] = pv_map[pv]
        return data

class IberdrolaMeasuresAdapter(MeasuresAdapter, MeasuresSchema):

    @pre_load
    def fix_dates(self, data):
        for attr, field in self.fields.iteritems():
            if isinstance(field, fields.DateTime):
                data[attr] += 'T00:00:00'
        return data


# -*- coding: utf-8 -*-
from marshmallow import Schema, fields


class HidrocantabricoSchema(Schema):
    ref = fields.String(description="Punto de suministro", position=0, length=10)
    name = fields.String(description="CUPS", position=1, length=22)
    distri = fields.String(description="Empresa distribuidora", position=2, length=50)
    cod_distri = fields.String(description="Codigo distribuidora", position=3, length=4)
    direccio = fields.String(description="Ubicación PS", position=4, length=81)
    poblacio = fields.String(description="Población", position=5, length=8)
    codi_postal = fields.String(description="Codigo Postal", position=6, length=5)
    provincia = fields.String(description="Provincia", position=7, length=2)
    data_alta = fields.String(description="Fecha de alta", position=8, length=8)
    tarifa = fields.String(description="Tarifa en vigor", position=9, length=5)
    tensio = fields.String(description="Tensión", position=10, length=10)
    pot_max_bie = fields.String(description="Potencia Máxima Boletín", position=11, length=10)
    pot_max_puesta = fields.String(description="Potencia Máxima APM", position=12, length=10)
    tipo_pm = fields.String(description="Tipo Punto Medida", position=13, length=2)
    indicatiu_icp = fields.String(description="Disponibilidad ICP", position=14, length=1)
    perfil_consum = fields.String(description="Tipo Perfil Consumo", position=15, length=1)
    der_extensio = fields.String(description="Derechos Extensión", position=16, length=10)
    der_acces_llano = fields.String(description="Derechos Acceso", position=17, length=10)
    propietat_equip_mesura = fields.String(description="Propiedad Equipo Medida", position=18, length=1)
    propietat_icp = fields.String(description="Propiedad ICP", position=19, length=1)
    pot_cont_p1 = fields.String(description="Potencias Contratada 1", position=20, length=10)
    pot_cont_p2 = fields.String(description="Potencias Contratada 2", position=21, length=10)
    pot_cont_p3 = fields.String(description="Potencias Contratada 3", position=22, length=10)
    pot_cont_p4 = fields.String(description="Potencias Contratada 4", position=23, length=10)
    pot_cont_p5 = fields.String(description="Potencias Contratada 5", position=24, length=10)
    pot_cont_p6 = fields.String(description="Potencias Contratada 6", position=25, length=10)
    potencias_contratada_7 = fields.String(description="Potencias Contratada 7", position=26, length=10)
    data_ulti_mov = fields.String(description="Fecha Ultimo Movimiento de Contratacion", position=27, length=8)
    data_ult_canv = fields.String(description="Fecha Ultimo Cambio de Comercializador", position=28, length=8)
    data_lim_exten = fields.String(description="Fecha límite derechos extensión", position=29, length=8)
    fecha_ultima_lectura = fields.String(description="Fecha ultima lectura", position=30, length=8)
    persona_fj = fields.String(description="Tipo de interlocutor", position=31, length=1)
    nombre_ic = fields.String(description="Nombre IC", position=32, length=80)
    ubicacion_ic = fields.String(description="Ubicación IC", position=33, length=81)
    poblacion_ic = fields.String(description="Población IC", position=34, length=12)
    codigo_postal = fields.String(description="Código postal", position=35, length=10)
    provincia_ = fields.String(description="Provincia", position=36, length=3)
    primera_vivenda = fields.String(description="vivienda habitual", position=37, length=1)
    fianza = fields.String(description="Depósito garantía", position=38, length=13)
    deuda = fields.String(description="Deuda", position=39, length=1)


class HidrocantabricoMeasuresSchema(Schema):
    name = fields.String(description="Suministro", position=0, length=10)
    fecha_inicio = fields.String(description="Fecha inicio", position=1, length=8)
    data_final = fields.String(description="Fecha fin", position=2, length=8)
    tarifa = fields.String(description="Tarifa", position=3, length=5)
    dh = fields.String(description="DH", position=4, length=1)
    activa_1 = fields.String(description="Activa 1", position=5, length=12)
    signo_a1 = fields.String(description="Signo", position=6, length=1)
    activa_2 = fields.String(description="Activa 2", position=7, length=12)
    signo_a2 = fields.String(description="Signo", position=8, length=1)
    activa_3 = fields.String(description="Activa 3", position=9, length=12)
    signo_a3 = fields.String(description="Signo", position=10, length=1)
    activa_4 = fields.String(description="Activa 4", position=11, length=12)
    signo_a4 = fields.String(description="Signo", position=12, length=1)
    activa_5 = fields.String(description="Activa 5", position=13, length=12)
    signo_a5 = fields.String(description="Signo", position=14, length=1)
    activa_6 = fields.String(description="Activa 6", position=15, length=12)
    signo_a6 = fields.String(description="Signo", position=16, length=1)
    activa_7 = fields.String(description="Activa 7", position=17, length=12)
    signo_a7 = fields.String(description="Signo", position=18, length=1)
    reactiva_1 = fields.String(description="Reactiva 1", position=19, length=12)
    signo_r1 = fields.String(description="Signo", position=20, length=1)
    reactiva_2 = fields.String(description="Reactiva 2", position=21, length=12)
    signo_r2 = fields.String(description="Signo", position=22, length=1)
    reactiva_3 = fields.String(description="Reactiva 3", position=23, length=12)
    signo_r3 = fields.String(description="Signo", position=24, length=1)
    reactiva_4 = fields.String(description="Reactiva 4", position=25, length=12)
    signo_r4 = fields.String(description="Signo", position=26, length=1)
    reactiva_5 = fields.String(description="Reactiva 5", position=27, length=12)
    signo_r5 = fields.String(description="Signo", position=28, length=1)
    reactiva_6 = fields.String(description="Reactiva 6", position=29, length=12)
    signo_r6 = fields.String(description="Signo", position=30, length=1)
    reactiva_7 = fields.String(description="Reactiva 7", position=31, length=12)
    signo_r7 = fields.String(description="Signo", position=32, length=1)
    potencia_1 = fields.String(description="Potencia 1", position=33, length=10)
    signo_p1 = fields.String(description="Signo", position=34, length=1)
    potencia_2 = fields.String(description="Potencia 2", position=35, length=10)
    signo_p2 = fields.String(description="Signo", position=36, length=1)
    potencia_3 = fields.String(description="Potencia 3", position=37, length=10)
    signo_p3 = fields.String(description="Signo", position=38, length=1)
    potencia_4 = fields.String(description="Potencia 4", position=39, length=10)
    signo_p4 = fields.String(description="Signo", position=40, length=1)
    potencia_5 = fields.String(description="Potencia 5", position=41, length=10)
    signo_p5 = fields.String(description="Signo", position=42, length=1)
    potencia_6 = fields.String(description="Potencia 6", position=43, length=10)
    signo_p6 = fields.String(description="Signo", position=44, length=1)
    potencia_7 = fields.String(description="Potencia 7", position=45, length=10)
    signo_p7 = fields.String(description="Signo", position=46, length=1)


from setuptools import setup, find_packages


setup(
    name='sippers',
    version='2.1.0',
    packages=find_packages(),
    url='https://github.com/gisce/sippers',
    license='GPLv3',
    author='GISCE-TI, S.L.',
    author_email='devel@gisce.net',
    description='sippers',
    long_description='sippers',
    entry_points='''
        [console_scripts]
        sippers=sippers.cli:sippers
    ''',
    package_data={
        'sippers': ['data/*']
    },
    install_requires=[
        "raven",
        "pymongo<3.0",
        "osconf",
        "marshmallow<3.0",
        "click"
    ],
    test_suite='tests',
)



from sippers import logger
from sippers.adapters import SipsAdapter, MeasuresAdapter
from sippers.models import SipsSchema, MeasuresSchema, TARIFFS as BASE_TARIFFS
from marshmallow import Schema, fields, pre_load


TARIFFS = {
    '20A': '2.0A',
    '20DHA': '2.0DHA',
    '20DHS': '2.0DHS',
    '21A': '2.1A',
    '21DHA': '2.1DHA',
    '21DHS': '2.1DHS',
    '30A': '3.0A',
    '31A': '3.1A',
    '61': '6.1',
    '61A': '6.1A',
    '61B': '6.1B',
    '62': '6.2',
    '63': '6.3',
    '64': '6.4',
    '65': '6.5'
}
"""Mapping of ENDESA tariffs codes."""


class EndesaBaseAdapter(Schema):
    """Endesa SIPS Adapter
    """

    @pre_load
    def fix_dates(self, data):
        """Fix the dates in the SIPS file from ENDESA.

        In the endesa sips file dates are ``YYYYMMDD`` or ``0`` or ``00000000``.
        With this all ``fields.DateTime`` fields are caugth and parsed to a
        correct format ``YYYY-MM-DDT00:00:00``.
        """
        for attr, field in self.fields.iteritems():
            if isinstance(field, fields.DateTime):
                orig = data.get(attr)
                if orig and orig not in ('0', '00000000'):
                    data[attr] = '{}-{}-{}T00:00:00'.format(
                        orig[0:4], orig[4:6], orig[6:8]
                    )
                else:
                    data[attr] = None
        return data

    @pre_load
    def fix_numbers(self, data):
        for attr, field in self.fields.iteritems():
            if isinstance(field, fields.Integer):
                if not data.get(attr):
                    data[attr] = 0
        return data

    @pre_load
    def fix_floats(self, data):
        """Fix floats numbers.

        Replace ``,`` to ``.``
        """
        for attr, field in self.fields.iteritems():
            if isinstance(field, fields.Float):
                if not data.get(attr):
                    data[attr] = 0
                if isinstance(data[attr], basestring):
                    data[attr] = data[attr].replace(',', '.')
        return data

class EndesaSipsAdapter(EndesaBaseAdapter, SipsAdapter, SipsSchema):
    """Endesa SIPS Adapter.
    """

    @pre_load
    def adapt_tarifa(self, data):
        """Fix the ATR Tariff code

        Using :attr:`TARIFFS`
        """
        tarifa = data.get('tarifa')
        if tarifa not in BASE_TARIFFS:
            tarifa = TARIFFS.get(tarifa, tarifa)
        data['tarifa'] = tarifa
        return data

    @pre_load
    def adapt_indicatiu_icp(self, data):
        icp = data.get('indicatiu_icp')
        if icp and icp == 'ICP INSTALADO':
            data['indicatiu_icp'] = '1'
        else:
            data['indicatiu_icp'] = '0'
        return data

    @pre_load
    def adapt_propietat_equip_mesura(self, data):
        propietat = data.get('propietat_equip_mesura')
        if propietat and propietat == 'EMPRESA DISTRIBUIDORA':
            data['propietat_equip_mesura'] = '1'
        else:
            data['propietat_equip_mesura'] = '0'
        return data

    @pre_load
    def adapt_perfil_consum(self, data):
        perfil = data.get('perfil_consum')
        if perfil:
            data['perfil_consum'] = data['perfil_consum'].title()
        else:
            data['perfil_consum'] = None
        return data

    @pre_load
    def adapt_primera_vivenda(self, data):
        vivenda = data.get('primera_vivenda')
        values = {'S': '1', 'N': '0'}
        if vivenda and vivenda in values:
            data['primera_vivenda'] = values[vivenda]
        else:
            data['primera_vivenda'] = None
        return data

    @pre_load
    def adapt_tipo_pm(self, data):
        pm = data.get('tipo_pm')
        values = {
            'TIPO I': '01',
            'TIPO II': '02',
            'TIPO III': '03',
            'TIPO IV': '04',
            'TIPO V': '05'
        }
        if pm and pm in values:
            data['tipo_pm'] = values[pm]
        else:
            data['tipo_pm'] = None
        return data

    @pre_load
    def adapt_persona_fj(self, data):
        persona = data.get('persona_fj')
        values = {'J': '1', 'F': '0'}
        if persona and persona in values:
            data['persona_fj'] = values[persona]
        else:
            data['persona_fj'] = None
        return data


class EndesaMeasuresAdapter(EndesaBaseAdapter, MeasuresSchema, MeasuresAdapter):
    pass
